"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MintInfo = exports.Bank = exports.QUOTE_DECIMALS = void 0;
const bytes_1 = require("@project-serum/anchor/dist/cjs/utils/bytes");
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../utils");
const I80F48_1 = require("./I80F48");
exports.QUOTE_DECIMALS = 6;
class Bank {
    constructor(publicKey, name, group, mint, vault, oracle, oracleConfig, depositIndex, borrowIndex, indexedTotalDeposits, indexedTotalBorrows, indexedDeposits, indexedBorrows, indexLastUpdated, bankRateLastUpdated, avgUtilization, adjustmentFactor, util0, rate0, util1, rate1, maxRate, collectedFeesNative, loanFeeRate, loanOriginationFeeRate, maintAssetWeight, initAssetWeight, maintLiabWeight, initLiabWeight, liquidationFee, dust, flashLoanTokenAccountInitial, flashLoanApprovedAmount, tokenIndex, mintDecimals, bankNum) {
        this.publicKey = publicKey;
        this.group = group;
        this.mint = mint;
        this.vault = vault;
        this.oracle = oracle;
        this.indexLastUpdated = indexLastUpdated;
        this.bankRateLastUpdated = bankRateLastUpdated;
        this.tokenIndex = tokenIndex;
        this.mintDecimals = mintDecimals;
        this.bankNum = bankNum;
        this.name = bytes_1.utf8.decode(new Uint8Array(name)).split('\x00')[0];
        this.depositIndex = I80F48_1.I80F48.from(depositIndex);
        this.borrowIndex = I80F48_1.I80F48.from(borrowIndex);
        this.indexedDeposits = I80F48_1.I80F48.from(indexedDeposits);
        this.indexedBorrows = I80F48_1.I80F48.from(indexedBorrows);
        this.cachedIndexedTotalDeposits = I80F48_1.I80F48.from(indexedTotalDeposits);
        this.cachedIndexedTotalBorrows = I80F48_1.I80F48.from(indexedTotalBorrows);
        this.avgUtilization = I80F48_1.I80F48.from(avgUtilization);
        this.adjustmentFactor = I80F48_1.I80F48.from(adjustmentFactor);
        this.maxRate = I80F48_1.I80F48.from(maxRate);
        this.util0 = I80F48_1.I80F48.from(util0);
        this.rate0 = I80F48_1.I80F48.from(rate0);
        this.util1 = I80F48_1.I80F48.from(util1);
        this.rate1 = I80F48_1.I80F48.from(rate1);
        this.collectedFeesNative = I80F48_1.I80F48.from(collectedFeesNative);
        this.loanFeeRate = I80F48_1.I80F48.from(loanFeeRate);
        this.loanOriginationFeeRate = I80F48_1.I80F48.from(loanOriginationFeeRate);
        this.maintAssetWeight = I80F48_1.I80F48.from(maintAssetWeight);
        this.initAssetWeight = I80F48_1.I80F48.from(initAssetWeight);
        this.maintLiabWeight = I80F48_1.I80F48.from(maintLiabWeight);
        this.initLiabWeight = I80F48_1.I80F48.from(initLiabWeight);
        this.liquidationFee = I80F48_1.I80F48.from(liquidationFee);
        this.dust = I80F48_1.I80F48.from(dust);
        this.price = undefined;
        this.uiPrice = undefined;
    }
    static from(publicKey, obj) {
        return new Bank(publicKey, obj.name, obj.group, obj.mint, obj.vault, obj.oracle, obj.oracleConfig, obj.depositIndex, obj.borrowIndex, obj.cachedIndexedTotalDeposits, obj.cachedIndexedTotalBorrows, obj.indexedDeposits, obj.indexedBorrows, obj.indexLastUpdated, obj.bankRateLastUpdated, obj.avgUtilization, obj.adjustmentFactor, obj.util0, obj.rate0, obj.util1, obj.rate1, obj.maxRate, obj.collectedFeesNative, obj.loanFeeRate, obj.loanOriginationFeeRate, obj.maintAssetWeight, obj.initAssetWeight, obj.maintLiabWeight, obj.initLiabWeight, obj.liquidationFee, obj.dust, obj.flashLoanTokenAccountInitial, obj.flashLoanApprovedAmount, obj.tokenIndex, obj.mintDecimals, obj.bankNum);
    }
    toString() {
        var _a;
        return ('Bank ' +
            '\n public key - ' +
            this.publicKey.toBase58() +
            '\n token index - ' +
            this.tokenIndex +
            '\n token name - ' +
            this.name +
            '\n vault - ' +
            this.vault.toBase58() +
            '\n mintDecimals - ' +
            this.mintDecimals +
            '\n oracle - ' +
            this.oracle.toBase58() +
            '\n price - ' +
            ((_a = this.price) === null || _a === void 0 ? void 0 : _a.toNumber()) +
            '\n uiPrice - ' +
            this.uiPrice +
            '\n deposit index - ' +
            this.depositIndex.toNumber() +
            '\n borrow index - ' +
            this.borrowIndex.toNumber() +
            '\n indexedDeposits - ' +
            this.indexedDeposits.toNumber() +
            '\n indexedBorrows - ' +
            this.indexedBorrows.toNumber() +
            '\n cachedIndexedTotalDeposits - ' +
            this.cachedIndexedTotalDeposits.toNumber() +
            '\n cachedIndexedTotalBorrows - ' +
            this.cachedIndexedTotalBorrows.toNumber() +
            '\n indexLastUpdated - ' +
            new Date(this.indexLastUpdated.toNumber() * 1000) +
            '\n bankRateLastUpdated - ' +
            new Date(this.bankRateLastUpdated.toNumber() * 1000) +
            '\n avgUtilization - ' +
            this.avgUtilization.toNumber() +
            '\n adjustmentFactor - ' +
            this.adjustmentFactor.toNumber() +
            '\n maxRate - ' +
            this.maxRate.toNumber() +
            '\n util0 - ' +
            this.util0.toNumber() +
            '\n rate0 - ' +
            this.rate0.toNumber() +
            '\n util1 - ' +
            this.util1.toNumber() +
            '\n rate1 - ' +
            this.rate1.toNumber() +
            '\n loanFeeRate - ' +
            this.loanFeeRate.toNumber() +
            '\n loanOriginationFeeRate - ' +
            this.loanOriginationFeeRate.toNumber() +
            '\n maintAssetWeight - ' +
            this.maintAssetWeight.toNumber() +
            '\n initAssetWeight - ' +
            this.initAssetWeight.toNumber() +
            '\n maintLiabWeight - ' +
            this.maintLiabWeight.toNumber() +
            '\n initLiabWeight - ' +
            this.initLiabWeight.toNumber() +
            '\n liquidationFee - ' +
            this.liquidationFee.toNumber() +
            '\n uiDeposits() - ' +
            this.uiDeposits() +
            '\n uiBorrows() - ' +
            this.uiBorrows() +
            '\n getDepositRate() - ' +
            this.getDepositRate().toNumber() +
            '\n getBorrowRate() - ' +
            this.getBorrowRate().toNumber());
    }
    nativeDeposits() {
        return this.indexedDeposits.mul(this.depositIndex);
    }
    nativeBorrows() {
        return this.indexedBorrows.mul(this.borrowIndex);
    }
    uiDeposits() {
        return (0, utils_1.nativeI80F48ToUi)(this.indexedDeposits.mul(this.depositIndex), this.mintDecimals).toNumber();
    }
    uiBorrows() {
        return (0, utils_1.nativeI80F48ToUi)(this.indexedBorrows.mul(this.borrowIndex), this.mintDecimals).toNumber();
    }
    /**
     *
     * @returns borrow rate, 0 is 0% where 1 is 100%
     */
    getBorrowRate() {
        const totalBorrows = this.nativeBorrows();
        const totalDeposits = this.nativeDeposits();
        if (totalDeposits.isZero() && totalBorrows.isZero()) {
            return (0, I80F48_1.ZERO_I80F48)();
        }
        if (totalDeposits.lte(totalBorrows)) {
            return this.maxRate;
        }
        const utilization = totalBorrows.div(totalDeposits);
        if (utilization.gt(this.util1)) {
            const extraUtil = utilization.sub(this.util1);
            const slope = this.maxRate
                .sub(this.rate1)
                .div(I80F48_1.I80F48.fromNumber(1).sub(this.util1));
            return this.rate1.add(slope.mul(extraUtil));
        }
        else if (utilization.gt(this.util0)) {
            const extraUtil = utilization.sub(this.util0);
            const slope = this.maxRate
                .sub(this.rate0)
                .div(I80F48_1.I80F48.fromNumber(1).sub(this.util0));
            return this.rate0.add(slope.mul(extraUtil));
        }
        else {
            const slope = this.rate0.div(this.util0);
            return slope.mul(utilization);
        }
    }
    /**
     *
     * @returns borrow rate percentage
     */
    getBorrowRateUi() {
        return this.getBorrowRate().toNumber() * 100;
    }
    /**
     *
     * @returns deposit rate, 0 is 0% where 1 is 100%
     */
    getDepositRate() {
        const borrowRate = this.getBorrowRate();
        const totalBorrows = this.nativeBorrows();
        const totalDeposits = this.nativeDeposits();
        if (totalDeposits.isZero() && totalBorrows.isZero()) {
            return (0, I80F48_1.ZERO_I80F48)();
        }
        else if (totalDeposits.isZero()) {
            return this.maxRate;
        }
        const utilization = totalBorrows.div(totalDeposits);
        return utilization.mul(borrowRate);
    }
    /**
     *
     * @returns deposit rate percentage
     */
    getDepositRateUi() {
        return this.getDepositRate().toNumber() * 100;
    }
}
exports.Bank = Bank;
class MintInfo {
    constructor(publicKey, group, tokenIndex, mint, banks, vaults, oracle, registrationTime, groupInsuranceFund) {
        this.publicKey = publicKey;
        this.group = group;
        this.tokenIndex = tokenIndex;
        this.mint = mint;
        this.banks = banks;
        this.vaults = vaults;
        this.oracle = oracle;
        this.registrationTime = registrationTime;
        this.groupInsuranceFund = groupInsuranceFund;
    }
    static from(publicKey, obj) {
        return new MintInfo(publicKey, obj.group, obj.tokenIndex, obj.mint, obj.banks, obj.vaults, obj.oracle, obj.registrationTime, obj.groupInsuranceFund);
    }
    firstBank() {
        return this.banks[0];
    }
    firstVault() {
        return this.vaults[0];
    }
    toString() {
        const res = 'mint ' +
            this.mint.toBase58() +
            '\n oracle ' +
            this.oracle.toBase58() +
            '\n banks ' +
            this.banks
                .filter((pk) => pk.toBase58() !== web3_js_1.PublicKey.default.toBase58())
                .toString() +
            '\n vaults ' +
            this.vaults
                .filter((pk) => pk.toBase58() !== web3_js_1.PublicKey.default.toBase58())
                .toString();
        return res;
    }
}
exports.MintInfo = MintInfo;
//# sourceMappingURL=bank.js.map