"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const anchor_1 = require("@project-serum/anchor");
const token_1 = require("@project-serum/anchor/dist/cjs/spl/token");
const serum_1 = require("@project-serum/serum");
const client_1 = require("@pythnetwork/client");
const constants_1 = require("../constants");
const ids_1 = require("../ids");
const utils_1 = require("../utils");
const bank_1 = require("./bank");
const I80F48_1 = require("./I80F48");
const oracle_1 = require("./oracle");
const perp_1 = require("./perp");
const serum3_1 = require("./serum3");
class Group {
    constructor(publicKey, creator, groupNum, admin, fastListingAdmin, insuranceMint, insuranceVault, testing, version, banksMapByName, banksMapByMint, banksMapByTokenIndex, serum3MarketsMapByExternal, serum3MarketExternalsMap, 
    // TODO rethink key
    perpMarketsMap, mintInfosMapByTokenIndex, mintInfosMapByMint, oraclesMap, // UNUSED
    vaultAmountsMap) {
        this.publicKey = publicKey;
        this.creator = creator;
        this.groupNum = groupNum;
        this.admin = admin;
        this.fastListingAdmin = fastListingAdmin;
        this.insuranceMint = insuranceMint;
        this.insuranceVault = insuranceVault;
        this.testing = testing;
        this.version = version;
        this.banksMapByName = banksMapByName;
        this.banksMapByMint = banksMapByMint;
        this.banksMapByTokenIndex = banksMapByTokenIndex;
        this.serum3MarketsMapByExternal = serum3MarketsMapByExternal;
        this.serum3MarketExternalsMap = serum3MarketExternalsMap;
        this.perpMarketsMap = perpMarketsMap;
        this.mintInfosMapByTokenIndex = mintInfosMapByTokenIndex;
        this.mintInfosMapByMint = mintInfosMapByMint;
        this.oraclesMap = oraclesMap;
        this.vaultAmountsMap = vaultAmountsMap;
    }
    static from(publicKey, obj) {
        return new Group(publicKey, obj.creator, obj.groupNum, obj.admin, obj.fastListingAdmin, obj.insuranceMint, obj.insuranceVault, obj.testing, obj.version, new Map(), // banksMapByName
        new Map(), // banksMapByMint
        new Map(), // banksMapByTokenIndex
        new Map(), // serum3MarketsMap
        new Map(), // serum3MarketExternalsMap
        new Map(), // perpMarketsMap
        new Map(), // mintInfosMapByTokenIndex
        new Map(), // mintInfosMapByMint
        new Map(), // oraclesMap
        new Map());
    }
    async reloadAll(client) {
        let ids = undefined;
        if (client.idsSource === 'api') {
            ids = await ids_1.Id.fromApi(this.publicKey);
        }
        else if (client.idsSource === 'static') {
            ids = ids_1.Id.fromIdsByPk(this.publicKey);
        }
        else {
            ids = undefined;
        }
        // console.time('group.reload');
        await Promise.all([
            this.reloadBanks(client, ids).then(() => Promise.all([
                this.reloadBankOraclePrices(client),
                this.reloadVaults(client, ids),
            ])),
            this.reloadMintInfos(client, ids),
            this.reloadSerum3Markets(client, ids).then(() => this.reloadSerum3ExternalMarkets(client, ids)),
            this.reloadPerpMarkets(client, ids).then(() => this.reloadPerpMarketOraclePrices(client)),
        ]);
        // console.timeEnd('group.reload');
    }
    async reloadBanks(client, ids) {
        var _a, _b, _c;
        let banks;
        if (ids && ids.getBanks().length) {
            banks = (await client.program.account.bank.fetchMultiple(ids.getBanks())).map((account, index) => bank_1.Bank.from(ids.getBanks()[index], account));
        }
        else {
            banks = await client.getBanksForGroup(this);
        }
        this.banksMapByName = new Map();
        this.banksMapByMint = new Map();
        this.banksMapByTokenIndex = new Map();
        for (const bank of banks) {
            const mintId = bank.mint.toString();
            if (this.banksMapByMint.has(mintId)) {
                (_a = this.banksMapByMint.get(mintId)) === null || _a === void 0 ? void 0 : _a.push(bank);
                (_b = this.banksMapByName.get(bank.name)) === null || _b === void 0 ? void 0 : _b.push(bank);
                (_c = this.banksMapByTokenIndex.get(bank.tokenIndex)) === null || _c === void 0 ? void 0 : _c.push(bank);
            }
            else {
                this.banksMapByMint.set(mintId, [bank]);
                this.banksMapByName.set(bank.name, [bank]);
                this.banksMapByTokenIndex.set(bank.tokenIndex, [bank]);
            }
        }
    }
    async reloadMintInfos(client, ids) {
        let mintInfos;
        if (ids && ids.getMintInfos().length) {
            mintInfos = (await client.program.account.mintInfo.fetchMultiple(ids.getMintInfos())).map((account, index) => bank_1.MintInfo.from(ids.getMintInfos()[index], account));
        }
        else {
            mintInfos = await client.getMintInfosForGroup(this);
        }
        this.mintInfosMapByTokenIndex = new Map(mintInfos.map((mintInfo) => {
            return [mintInfo.tokenIndex, mintInfo];
        }));
        this.mintInfosMapByMint = new Map(mintInfos.map((mintInfo) => {
            return [mintInfo.mint.toString(), mintInfo];
        }));
    }
    async reloadSerum3Markets(client, ids) {
        let serum3Markets;
        if (ids && ids.getSerum3Markets().length) {
            serum3Markets = (await client.program.account.serum3Market.fetchMultiple(ids.getSerum3Markets())).map((account, index) => serum3_1.Serum3Market.from(ids.getSerum3Markets()[index], account));
        }
        else {
            serum3Markets = await client.serum3GetMarkets(this);
        }
        this.serum3MarketsMapByExternal = new Map(serum3Markets.map((serum3Market) => [
            serum3Market.serumMarketExternal.toBase58(),
            serum3Market,
        ]));
    }
    async reloadSerum3ExternalMarkets(client, ids) {
        const externalMarkets = await Promise.all(Array.from(this.serum3MarketsMapByExternal.values()).map((serum3Market) => serum_1.Market.load(client.program.provider.connection, serum3Market.serumMarketExternal, { commitment: client.program.provider.connection.commitment }, constants_1.SERUM3_PROGRAM_ID[client.cluster])));
        this.serum3MarketExternalsMap = new Map(Array.from(this.serum3MarketsMapByExternal.values()).map((serum3Market, index) => [
            serum3Market.serumMarketExternal.toBase58(),
            externalMarkets[index],
        ]));
    }
    async reloadPerpMarkets(client, ids) {
        let perpMarkets;
        if (ids && ids.getPerpMarkets().length) {
            perpMarkets = (await client.program.account.perpMarket.fetchMultiple(ids.getPerpMarkets())).map((account, index) => perp_1.PerpMarket.from(ids.getPerpMarkets()[index], account));
        }
        else {
            perpMarkets = await client.perpGetMarkets(this);
        }
        this.perpMarketsMap = new Map(perpMarkets.map((perpMarket) => [perpMarket.name, perpMarket]));
    }
    async reloadBankOraclePrices(client) {
        const banks = Array.from(this.banksMapByMint, ([, value]) => value);
        const oracles = banks.map((b) => b[0].oracle);
        const ais = await client.program.provider.connection.getMultipleAccountsInfo(oracles);
        const coder = new anchor_1.BorshAccountsCoder(client.program.idl);
        for (const [index, ai] of ais.entries()) {
            for (const bank of banks[index]) {
                if (bank.name === 'USDC') {
                    bank.price = (0, I80F48_1.ONE_I80F48)();
                    bank.uiPrice = 1;
                }
                else {
                    if (!ai)
                        throw new Error(`Undefined accountInfo object in reloadBankOraclePrices for ${bank.oracle}!`);
                    const { price, uiPrice } = await this.decodePriceFromOracleAi(coder, bank.oracle, ai, this.getMintDecimals(bank.mint), this.getMintDecimals(this.insuranceMint));
                    bank.price = price;
                    bank.uiPrice = uiPrice;
                }
            }
        }
    }
    async reloadPerpMarketOraclePrices(client) {
        const perpMarkets = Array.from(this.perpMarketsMap.values());
        const oracles = perpMarkets.map((b) => b.oracle);
        const ais = await client.program.provider.connection.getMultipleAccountsInfo(oracles);
        const coder = new anchor_1.BorshAccountsCoder(client.program.idl);
        ais.forEach(async (ai, i) => {
            const perpMarket = perpMarkets[i];
            if (!ai)
                throw new Error('Undefined ai object in reloadPerpMarketOraclePrices!');
            const { price, uiPrice } = await this.decodePriceFromOracleAi(coder, perpMarket.oracle, ai, perpMarket.baseDecimals, this.getMintDecimals(this.insuranceMint));
            perpMarket.price = price;
            perpMarket.uiPrice = uiPrice;
        });
    }
    async decodePriceFromOracleAi(coder, oracle, ai, baseDecimals, quoteDecimals) {
        let price, uiPrice;
        if (!anchor_1.BorshAccountsCoder.accountDiscriminator('stubOracle').compare(ai.data.slice(0, 8))) {
            const stubOracle = coder.decode('stubOracle', ai.data);
            price = new I80F48_1.I80F48(stubOracle.price.val);
            uiPrice = this === null || this === void 0 ? void 0 : this.toUiPrice(price, baseDecimals, quoteDecimals);
        }
        else if ((0, oracle_1.isPythOracle)(ai)) {
            uiPrice = (0, client_1.parsePriceData)(ai.data).previousPrice;
            price = this === null || this === void 0 ? void 0 : this.toNativePrice(uiPrice, baseDecimals, quoteDecimals);
        }
        else if ((0, oracle_1.isSwitchboardOracle)(ai)) {
            uiPrice = await (0, oracle_1.parseSwitchboardOracle)(ai);
            price = this === null || this === void 0 ? void 0 : this.toNativePrice(uiPrice, baseDecimals, quoteDecimals);
        }
        else {
            throw new Error(`Unknown oracle provider for oracle ${oracle}, with owner ${ai.owner}`);
        }
        return { price, uiPrice };
    }
    async reloadVaults(client, ids) {
        const vaultPks = Array.from(this.banksMapByMint.values())
            .flat()
            .map((bank) => bank.vault);
        const vaultAccounts = await client.program.provider.connection.getMultipleAccountsInfo(vaultPks);
        this.vaultAmountsMap = new Map(vaultAccounts.map((vaultAi, i) => {
            if (!vaultAi)
                throw new Error('Missing vault account info');
            const vaultAmount = (0, token_1.coder)()
                .accounts.decode('token', vaultAi.data)
                .amount.toNumber();
            return [vaultPks[i].toBase58(), vaultAmount];
        }));
    }
    getMintDecimals(mintPk) {
        const banks = this.banksMapByMint.get(mintPk.toString());
        if (!banks)
            throw new Error(`Unable to find mint decimals for ${mintPk.toString()}`);
        return banks[0].mintDecimals;
    }
    getFirstBankByMint(mintPk) {
        const banks = this.banksMapByMint.get(mintPk.toString());
        if (!banks)
            throw new Error(`Unable to find bank for ${mintPk.toString()}`);
        return banks[0];
    }
    getFirstBankByTokenIndex(tokenIndex) {
        const banks = this.banksMapByTokenIndex.get(tokenIndex);
        if (!banks)
            throw new Error(`Unable to find banks for tokenIndex ${tokenIndex}`);
        return banks[0];
    }
    /**
     *
     * @param mintPk
     * @returns sum of native balances of vaults for all banks for a token (fetched from vaultAmountsMap cache)
     */
    getTokenVaultBalanceByMint(mintPk) {
        const banks = this.banksMapByMint.get(mintPk.toBase58());
        if (!banks)
            throw new Error(`Mint does not exist in getTokenVaultBalanceByMint ${mintPk.toString()}`);
        let totalAmount = 0;
        for (const bank of banks) {
            const amount = this.vaultAmountsMap.get(bank.vault.toBase58());
            if (amount) {
                totalAmount += amount;
            }
        }
        return I80F48_1.I80F48.fromNumber(totalAmount);
    }
    getSerum3MarketByPk(pk) {
        return Array.from(this.serum3MarketsMapByExternal.values()).find((serum3Market) => serum3Market.serumMarketExternal.equals(pk));
    }
    getSerum3MarketByIndex(marketIndex) {
        return Array.from(this.serum3MarketsMapByExternal.values()).find((serum3Market) => serum3Market.marketIndex === marketIndex);
    }
    getSerum3MarketByName(name) {
        return Array.from(this.serum3MarketsMapByExternal.values()).find((serum3Market) => serum3Market.name === name);
    }
    async loadSerum3BidsForMarket(client, externalMarketPk) {
        const serum3Market = this.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        return await serum3Market.loadBids(client, this);
    }
    async loadSerum3AsksForMarket(client, externalMarketPk) {
        const serum3Market = this.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        return await serum3Market.loadAsks(client, this);
    }
    getFeeRate(maker = true) {
        // TODO: fetch msrm/srm vault balance
        const feeTier = (0, serum_1.getFeeTier)(0, 0);
        const rates = (0, serum_1.getFeeRates)(feeTier);
        return maker ? rates.maker : rates.taker;
    }
    async loadPerpBidsForMarket(client, marketName) {
        const perpMarket = this.perpMarketsMap.get(marketName);
        if (!perpMarket) {
            throw new Error(`Perp Market ${marketName} not found!`);
        }
        return await perpMarket.loadBids(client);
    }
    async loadPerpAsksForMarket(client, marketName) {
        const perpMarket = this.perpMarketsMap.get(marketName);
        if (!perpMarket) {
            throw new Error(`Perp Market ${marketName} not found!`);
        }
        return await perpMarket.loadAsks(client);
    }
    /**
     *
     * @param mintPk
     * @returns sum of ui balances of vaults for all banks for a token
     */
    getTokenVaultBalanceByMintUi(mintPk) {
        const vaultBalance = this.getTokenVaultBalanceByMint(mintPk);
        const mintDecimals = this.getMintDecimals(mintPk);
        return (0, utils_1.toUiDecimals)(vaultBalance, mintDecimals);
    }
    consoleLogBanks() {
        for (const mintBanks of this.banksMapByMint.values()) {
            for (const bank of mintBanks) {
                console.log(bank.toString());
            }
        }
    }
    toUiPrice(price, baseDecimals, quoteDecimals) {
        return price
            .mul(I80F48_1.I80F48.fromNumber(Math.pow(10, baseDecimals - quoteDecimals)))
            .toNumber();
    }
    toNativePrice(uiPrice, baseDecimals, quoteDecimals) {
        return I80F48_1.I80F48.fromNumber(uiPrice).mul(I80F48_1.I80F48.fromNumber(Math.pow(10, quoteDecimals - baseDecimals)));
    }
    toNativeDecimals(uiAmount, mintPk) {
        const decimals = this.getMintDecimals(mintPk);
        return (0, utils_1.toNativeDecimals)(uiAmount, decimals);
    }
    toString() {
        let res = 'Group\n';
        res = res + ' pk: ' + this.publicKey.toString();
        res =
            res +
                '\n mintInfos:' +
                Array.from(this.mintInfosMapByTokenIndex.entries())
                    .map((mintInfoTuple) => '  \n' + mintInfoTuple[0] + ') ' + mintInfoTuple[1].toString())
                    .join(', ');
        const banks = [];
        for (const tokenBanks of this.banksMapByMint.values()) {
            for (const bank of tokenBanks) {
                banks.push(bank);
            }
        }
        res =
            res +
                '\n banks:' +
                Array.from(banks)
                    .map((bank) => '  \n' + bank.name + ') ' + bank.toString())
                    .join(', ');
        return res;
    }
}
exports.Group = Group;
//# sourceMappingURL=group.js.map