"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPythOracle = exports.isSwitchboardOracle = exports.parseSwitchboardOracle = exports.parseSwitcboardOracleV2 = exports.parseSwitcboardOracleV1 = exports.StubOracle = void 0;
const client_1 = require("@pythnetwork/client");
const web3_js_1 = require("@solana/web3.js");
const switchboard_v2_1 = require("@switchboard-xyz/switchboard-v2");
const I80F48_1 = require("./I80F48");
const SBV1_DEVNET_PID = new web3_js_1.PublicKey('7azgmy1pFXHikv36q1zZASvFq5vFa39TT9NweVugKKTU');
const SBV1_MAINNET_PID = new web3_js_1.PublicKey('DtmE9D2CSB4L5D6A15mraeEjrGMm6auWVzgaD8hK2tZM');
let sbv2DevnetProgram;
let sbv2MainnetProgram;
class StubOracle {
    constructor(publicKey, group, mint, price, lastUpdated) {
        this.publicKey = publicKey;
        this.group = group;
        this.mint = mint;
        this.price = I80F48_1.I80F48.from(price);
        this.lastUpdated = lastUpdated.toNumber();
    }
    static from(publicKey, obj) {
        return new StubOracle(publicKey, obj.group, obj.mint, obj.price, obj.lastUpdated);
    }
}
exports.StubOracle = StubOracle;
// https://gist.github.com/microwavedcola1/b741a11e6ee273a859f3ef00b35ac1f0
function parseSwitcboardOracleV1(accountInfo) {
    return accountInfo.data.readDoubleLE(1 + 32 + 4 + 4);
}
exports.parseSwitcboardOracleV1 = parseSwitcboardOracleV1;
function parseSwitcboardOracleV2(program, accountInfo) {
    const aggregatorAccountData = program._coder.accounts.decode(program.account.aggregatorAccountData._idlAccount.name, accountInfo.data);
    const sbDecimal = switchboard_v2_1.SwitchboardDecimal.from(aggregatorAccountData.latestConfirmedRound.result);
    return sbDecimal.toBig().toNumber();
}
exports.parseSwitcboardOracleV2 = parseSwitcboardOracleV2;
/**
 *
 * @param accountInfo
 * @returns ui price
 */
async function parseSwitchboardOracle(accountInfo) {
    if (accountInfo.owner.equals(switchboard_v2_1.SBV2_DEVNET_PID)) {
        if (!sbv2DevnetProgram) {
            sbv2DevnetProgram = await (0, switchboard_v2_1.loadSwitchboardProgram)('devnet');
        }
        return parseSwitcboardOracleV2(sbv2DevnetProgram, accountInfo);
    }
    if (accountInfo.owner.equals(switchboard_v2_1.SBV2_MAINNET_PID)) {
        if (!sbv2MainnetProgram) {
            sbv2MainnetProgram = await (0, switchboard_v2_1.loadSwitchboardProgram)('mainnet-beta');
        }
        return parseSwitcboardOracleV2(sbv2MainnetProgram, accountInfo);
    }
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID)) {
        return parseSwitcboardOracleV1(accountInfo);
    }
    throw new Error(`Unable to parse switchboard oracle ${accountInfo.owner}`);
}
exports.parseSwitchboardOracle = parseSwitchboardOracle;
function isSwitchboardOracle(accountInfo) {
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID) ||
        accountInfo.owner.equals(switchboard_v2_1.SBV2_DEVNET_PID) ||
        accountInfo.owner.equals(switchboard_v2_1.SBV2_MAINNET_PID)) {
        return true;
    }
    return false;
}
exports.isSwitchboardOracle = isSwitchboardOracle;
function isPythOracle(accountInfo) {
    return accountInfo.data.readUInt32LE(0) === client_1.Magic;
}
exports.isPythOracle = isPythOracle;
//# sourceMappingURL=oracle.js.map