"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nativeI80F48ToUi = exports.toU64 = exports.toUiDecimalsForQuote = exports.toUiDecimals = exports.toNativeDecimals = exports.toNative = exports.createAssociatedTokenAccountIdempotentInstruction = exports.getAssociatedTokenAddress = exports.findOrCreate = exports.debugHealthAccounts = exports.debugAccountMetas = exports.I64_MAX_BN = exports.U64_MAX_BN = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js"));
const bank_1 = require("./accounts/bank");
const I80F48_1 = require("./accounts/I80F48");
exports.U64_MAX_BN = new bn_js_1.default('18446744073709551615');
exports.I64_MAX_BN = new bn_js_1.default('9223372036854775807').toTwos(64);
function debugAccountMetas(ams) {
    for (const am of ams) {
        console.log(`${am.pubkey.toBase58()}, isSigner: ${am.isSigner
            .toString()
            .padStart(5, ' ')}, isWritable - ${am.isWritable
            .toString()
            .padStart(5, ' ')}`);
    }
}
exports.debugAccountMetas = debugAccountMetas;
function debugHealthAccounts(group, mangoAccount, publicKeys) {
    const banks = new Map(Array.from(group.banksMapByName.values()).map((banks) => [
        banks[0].publicKey.toBase58(),
        `${banks[0].name} bank`,
    ]));
    const oracles = new Map(Array.from(group.banksMapByName.values()).map((banks) => [
        banks[0].oracle.toBase58(),
        `${banks[0].name} oracle`,
    ]));
    const serum3 = new Map(mangoAccount.serum3Active().map((serum3) => {
        const serum3Market = Array.from(group.serum3MarketsMapByExternal.values()).find((serum3Market) => serum3Market.marketIndex === serum3.marketIndex);
        if (!serum3Market) {
            throw new Error(`Serum3Orders for non existent market with market index ${serum3.marketIndex}`);
        }
        return [serum3.openOrders.toBase58(), `${serum3Market.name} spot oo`];
    }));
    const perps = new Map(Array.from(group.perpMarketsMap.values()).map((perpMarket) => [
        perpMarket.publicKey.toBase58(),
        `${perpMarket.name} perp market`,
    ]));
    publicKeys.map((pk) => {
        if (banks.get(pk.toBase58())) {
            console.log(banks.get(pk.toBase58()));
        }
        if (oracles.get(pk.toBase58())) {
            console.log(oracles.get(pk.toBase58()));
        }
        if (serum3.get(pk.toBase58())) {
            console.log(serum3.get(pk.toBase58()));
        }
        if (perps.get(pk.toBase58())) {
            console.log(perps.get(pk.toBase58()));
        }
    });
}
exports.debugHealthAccounts = debugHealthAccounts;
async function findOrCreate(entityName, findMethod, findArgs, createMethod, createArgs) {
    let many = await findMethod(...findArgs);
    let one;
    if (many.length > 0) {
        one = many[0];
        return one;
    }
    await createMethod(...createArgs);
    many = await findMethod(...findArgs);
    one = many[0];
    return one;
}
exports.findOrCreate = findOrCreate;
/**
 * Get the address of the associated token account for a given mint and owner
 *
 * @param mint                     Token mint account
 * @param owner                    Owner of the new account
 * @param allowOwnerOffCurve       Allow the owner account to be a PDA (Program Derived Address)
 * @param programId                SPL Token program account
 * @param associatedTokenProgramId SPL Associated Token program account
 *
 * @return Address of the associated token account
 */
async function getAssociatedTokenAddress(mint, owner, allowOwnerOffCurve = false, programId = spl_token_1.TOKEN_PROGRAM_ID, associatedTokenProgramId = spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID) {
    if (!allowOwnerOffCurve && !web3_js_1.PublicKey.isOnCurve(owner.toBuffer()))
        throw new Error('TokenOwnerOffCurve');
    const [address] = await web3_js_1.PublicKey.findProgramAddress([owner.toBuffer(), programId.toBuffer(), mint.toBuffer()], associatedTokenProgramId);
    return address;
}
exports.getAssociatedTokenAddress = getAssociatedTokenAddress;
async function createAssociatedTokenAccountIdempotentInstruction(payer, owner, mint) {
    const account = await getAssociatedTokenAddress(mint, owner);
    return new web3_js_1.TransactionInstruction({
        keys: [
            { pubkey: payer, isSigner: true, isWritable: true },
            { pubkey: account, isSigner: false, isWritable: true },
            { pubkey: owner, isSigner: false, isWritable: false },
            { pubkey: mint, isSigner: false, isWritable: false },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            { pubkey: spl_token_1.TOKEN_PROGRAM_ID, isSigner: false, isWritable: false },
        ],
        programId: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
        data: Buffer.from([0x1]),
    });
}
exports.createAssociatedTokenAccountIdempotentInstruction = createAssociatedTokenAccountIdempotentInstruction;
function toNative(uiAmount, decimals) {
    return I80F48_1.I80F48.fromNumber(uiAmount).mul(I80F48_1.I80F48.fromNumber(Math.pow(10, decimals)));
}
exports.toNative = toNative;
function toNativeDecimals(amount, decimals) {
    return new bn_js_1.default(Math.trunc(amount * Math.pow(10, decimals)));
}
exports.toNativeDecimals = toNativeDecimals;
function toUiDecimals(amount, decimals) {
    amount = amount instanceof I80F48_1.I80F48 ? amount.toNumber() : amount;
    return amount / Math.pow(10, decimals);
}
exports.toUiDecimals = toUiDecimals;
function toUiDecimalsForQuote(amount) {
    amount = amount instanceof I80F48_1.I80F48 ? amount.toNumber() : amount;
    return amount / Math.pow(10, bank_1.QUOTE_DECIMALS);
}
exports.toUiDecimalsForQuote = toUiDecimalsForQuote;
function toU64(amount, decimals) {
    const bn = toNativeDecimals(amount, decimals).toString();
    console.log('bn', bn);
    return new bn_js_1.default(bn);
}
exports.toU64 = toU64;
function nativeI80F48ToUi(amount, decimals) {
    return amount.div(I80F48_1.I80F48.fromNumber(Math.pow(10, decimals)));
}
exports.nativeI80F48ToUi = nativeI80F48ToUi;
//# sourceMappingURL=utils.js.map