export async function sendTransaction(provider, transaction, opts = {}) {
    var _a;
    const connection = provider.connection;
    const payer = provider.wallet;
    const latestBlockhash = await connection.getLatestBlockhash(opts.preflightCommitment);
    transaction.recentBlockhash = latestBlockhash.blockhash;
    transaction.lastValidBlockHeight = latestBlockhash.lastValidBlockHeight;
    transaction.feePayer = payer.publicKey;
    if (((_a = opts.additionalSigners) === null || _a === void 0 ? void 0 : _a.length) > 0) {
        transaction.partialSign(...opts.additionalSigners);
    }
    await payer.signTransaction(transaction);
    const rawTransaction = transaction.serialize();
    const signature = await connection.sendRawTransaction(rawTransaction, {
        skipPreflight: true,
    });
    if (opts.postSendTxCallback) {
        try {
            opts.postSendTxCallback({ txid: signature });
        }
        catch (e) {
            console.warn(`postSendTxCallback error ${e}`);
        }
    }
    let status;
    if (transaction.recentBlockhash != null &&
        transaction.lastValidBlockHeight != null) {
        console.log('confirming via blockhash');
        status = (await connection.confirmTransaction({
            signature: signature,
            blockhash: transaction.recentBlockhash,
            lastValidBlockHeight: transaction.lastValidBlockHeight,
        }, 'processed')).value;
    }
    else {
        console.log('confirming via timeout');
        status = (await connection.confirmTransaction(signature, 'processed'))
            .value;
    }
    if (status.err) {
        throw new MangoError({
            txid: signature,
            message: `Transaction ${signature} failed (${JSON.stringify(status)})`,
        });
    }
    return signature;
}
class MangoError extends Error {
    constructor({ txid, message }) {
        super();
        this.message = message;
        this.txid = txid;
    }
}
//# sourceMappingURL=rpc.js.map