import { Market, Orderbook } from '@project-serum/serum';
import { PriceData } from '@pythnetwork/client';
import { PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import { MangoClient } from '../client';
import { Id } from '../ids';
import { Bank, MintInfo } from './bank';
import { I80F48 } from './I80F48';
import { BookSide, PerpMarket } from './perp';
import { Serum3Market } from './serum3';
export declare class Group {
    publicKey: PublicKey;
    creator: PublicKey;
    groupNum: number;
    admin: PublicKey;
    fastListingAdmin: PublicKey;
    insuranceMint: PublicKey;
    insuranceVault: PublicKey;
    testing: number;
    version: number;
    banksMapByName: Map<string, Bank[]>;
    banksMapByMint: Map<string, Bank[]>;
    banksMapByTokenIndex: Map<number, Bank[]>;
    serum3MarketsMapByExternal: Map<string, Serum3Market>;
    serum3MarketExternalsMap: Map<string, Market>;
    perpMarketsMap: Map<string, PerpMarket>;
    mintInfosMapByTokenIndex: Map<number, MintInfo>;
    mintInfosMapByMint: Map<string, MintInfo>;
    private oraclesMap;
    vaultAmountsMap: Map<string, number>;
    static from(publicKey: PublicKey, obj: {
        creator: PublicKey;
        groupNum: number;
        admin: PublicKey;
        fastListingAdmin: PublicKey;
        insuranceMint: PublicKey;
        insuranceVault: PublicKey;
        testing: number;
        version: number;
    }): Group;
    constructor(publicKey: PublicKey, creator: PublicKey, groupNum: number, admin: PublicKey, fastListingAdmin: PublicKey, insuranceMint: PublicKey, insuranceVault: PublicKey, testing: number, version: number, banksMapByName: Map<string, Bank[]>, banksMapByMint: Map<string, Bank[]>, banksMapByTokenIndex: Map<number, Bank[]>, serum3MarketsMapByExternal: Map<string, Serum3Market>, serum3MarketExternalsMap: Map<string, Market>, perpMarketsMap: Map<string, PerpMarket>, mintInfosMapByTokenIndex: Map<number, MintInfo>, mintInfosMapByMint: Map<string, MintInfo>, oraclesMap: Map<string, PriceData>, // UNUSED
    vaultAmountsMap: Map<string, number>);
    reloadAll(client: MangoClient): Promise<void>;
    reloadBanks(client: MangoClient, ids?: Id): Promise<void>;
    reloadMintInfos(client: MangoClient, ids?: Id): Promise<void>;
    reloadSerum3Markets(client: MangoClient, ids?: Id): Promise<void>;
    reloadSerum3ExternalMarkets(client: MangoClient, ids?: Id): Promise<void>;
    reloadPerpMarkets(client: MangoClient, ids?: Id): Promise<void>;
    reloadBankOraclePrices(client: MangoClient): Promise<void>;
    reloadPerpMarketOraclePrices(client: MangoClient): Promise<void>;
    private decodePriceFromOracleAi;
    reloadVaults(client: MangoClient, ids?: Id): Promise<void>;
    getMintDecimals(mintPk: PublicKey): number;
    getFirstBankByMint(mintPk: PublicKey): Bank;
    getFirstBankByTokenIndex(tokenIndex: number): Bank;
    /**
     *
     * @param mintPk
     * @returns sum of native balances of vaults for all banks for a token (fetched from vaultAmountsMap cache)
     */
    getTokenVaultBalanceByMint(mintPk: PublicKey): I80F48;
    getSerum3MarketByPk(pk: PublicKey): Serum3Market | undefined;
    getSerum3MarketByIndex(marketIndex: number): Serum3Market | undefined;
    getSerum3MarketByName(name: string): Serum3Market | undefined;
    loadSerum3BidsForMarket(client: MangoClient, externalMarketPk: PublicKey): Promise<Orderbook>;
    loadSerum3AsksForMarket(client: MangoClient, externalMarketPk: PublicKey): Promise<Orderbook>;
    getFeeRate(maker?: boolean): number;
    loadPerpBidsForMarket(client: MangoClient, marketName: string): Promise<BookSide>;
    loadPerpAsksForMarket(client: MangoClient, marketName: string): Promise<BookSide>;
    /**
     *
     * @param mintPk
     * @returns sum of ui balances of vaults for all banks for a token
     */
    getTokenVaultBalanceByMintUi(mintPk: PublicKey): number;
    consoleLogBanks(): void;
    toUiPrice(price: I80F48, baseDecimals: number, quoteDecimals: number): number;
    toNativePrice(uiPrice: number, baseDecimals: number, quoteDecimals: number): I80F48;
    toNativeDecimals(uiAmount: number, mintPk: PublicKey): BN;
    toString(): string;
}
