import { PublicKey } from '@solana/web3.js';
import { Bank } from './bank';
import { Group } from './group';
import { I80F48, I80F48Dto } from './I80F48';
import { HealthType } from './mangoAccount';
import { Serum3Market, Serum3Side } from './serum3';
export declare class HealthCache {
    tokenInfos: TokenInfo[];
    serum3Infos: Serum3Info[];
    perpInfos: PerpInfo[];
    constructor(tokenInfos: TokenInfo[], serum3Infos: Serum3Info[], perpInfos: PerpInfo[]);
    static fromDto(dto: any): HealthCache;
    health(healthType: HealthType): I80F48;
    assets(healthType: HealthType): I80F48;
    liabs(healthType: HealthType): I80F48;
    healthRatio(healthType: HealthType): I80F48;
    findTokenInfoIndex(tokenIndex: number): number;
    getOrCreateTokenInfoIndex(bank: Bank): number;
    findSerum3InfoIndex(marketIndex: number): number;
    getOrCreateSerum3InfoIndex(group: Group, serum3Market: Serum3Market): number;
    adjustSerum3Reserved(group: Group, serum3Market: Serum3Market, reservedBaseChange: I80F48, freeBaseChange: I80F48, reservedQuoteChange: I80F48, freeQuoteChange: I80F48): void;
    static logHealthCache(debug: string, healthCache: HealthCache): void;
    simHealthRatioWithTokenPositionChanges(group: Group, nativeTokenChanges: {
        nativeTokenAmount: I80F48;
        mintPk: PublicKey;
    }[], healthType?: HealthType): I80F48;
    simHealthRatioWithSerum3BidChanges(group: Group, bidNativeQuoteAmount: I80F48, serum3Market: Serum3Market, healthType?: HealthType): I80F48;
    simHealthRatioWithSerum3AskChanges(group: Group, askNativeBaseAmount: I80F48, serum3Market: Serum3Market, healthType?: HealthType): I80F48;
    private static binaryApproximationSearch;
    getMaxSourceForTokenSwap(group: Group, sourceMintPk: PublicKey, targetMintPk: PublicKey, minRatio: I80F48): I80F48;
    getMaxForSerum3Order(group: Group, serum3Market: Serum3Market, side: Serum3Side, minRatio: I80F48): any;
}
export declare class TokenInfo {
    tokenIndex: number;
    maintAssetWeight: I80F48;
    initAssetWeight: I80F48;
    maintLiabWeight: I80F48;
    initLiabWeight: I80F48;
    oraclePrice: I80F48;
    balance: I80F48;
    serum3MaxReserved: I80F48;
    constructor(tokenIndex: number, maintAssetWeight: I80F48, initAssetWeight: I80F48, maintLiabWeight: I80F48, initLiabWeight: I80F48, oraclePrice: I80F48, balance: I80F48, serum3MaxReserved: I80F48);
    static fromDto(dto: TokenInfoDto): TokenInfo;
    static emptyFromBank(bank: Bank): TokenInfo;
    assetWeight(healthType: HealthType): I80F48;
    liabWeight(healthType: HealthType): I80F48;
    healthContribution(healthType: HealthType): I80F48;
    toString(): string;
}
export declare class Serum3Info {
    reserved: I80F48;
    baseIndex: number;
    quoteIndex: number;
    marketIndex: number;
    constructor(reserved: I80F48, baseIndex: number, quoteIndex: number, marketIndex: number);
    static fromDto(dto: Serum3InfoDto): Serum3Info;
    static emptyFromSerum3Market(serum3Market: Serum3Market, baseEntryIndex: number, quoteEntryIndex: number): Serum3Info;
    healthContribution(healthType: HealthType, tokenInfos: TokenInfo[]): I80F48;
    toString(tokenInfos: TokenInfo[]): string;
}
export declare class PerpInfo {
    constructor(dto: PerpInfoDto);
    maintAssetWeight: I80F48;
    initAssetWeight: I80F48;
    maintLiabWeight: I80F48;
    initLiabWeight: I80F48;
    base: I80F48;
    quote: I80F48;
    healthContribution(healthType: HealthType): I80F48;
}
export declare class HealthCacheDto {
    tokenInfos: TokenInfoDto[];
    serum3Infos: Serum3InfoDto[];
    perpInfos: PerpInfoDto[];
}
export declare class TokenInfoDto {
    tokenIndex: number;
    maintAssetWeight: I80F48Dto;
    initAssetWeight: I80F48Dto;
    maintLiabWeight: I80F48Dto;
    initLiabWeight: I80F48Dto;
    oraclePrice: I80F48Dto;
    balance: I80F48Dto;
    serum3MaxReserved: I80F48Dto;
    constructor(tokenIndex: number, maintAssetWeight: I80F48Dto, initAssetWeight: I80F48Dto, maintLiabWeight: I80F48Dto, initLiabWeight: I80F48Dto, oraclePrice: I80F48Dto, balance: I80F48Dto, serum3MaxReserved: I80F48Dto);
}
export declare class Serum3InfoDto {
    reserved: I80F48Dto;
    baseIndex: number;
    quoteIndex: number;
    marketIndex: number;
    constructor(reserved: I80F48Dto, baseIndex: number, quoteIndex: number);
}
export declare class PerpInfoDto {
    maintAssetWeight: I80F48Dto;
    initAssetWeight: I80F48Dto;
    maintLiabWeight: I80F48Dto;
    initLiabWeight: I80F48Dto;
    base: I80F48Dto;
    quote: I80F48Dto;
}
