/// <reference types="bn.js" />
import { BN } from '@project-serum/anchor';
import { PublicKey } from '@solana/web3.js';
import { MangoClient } from '../client';
import { OracleConfig } from './bank';
import { I80F48, I80F48Dto } from './I80F48';
export declare class PerpMarket {
    publicKey: PublicKey;
    group: PublicKey;
    quoteTokenIndex: number;
    perpMarketIndex: number;
    oracle: PublicKey;
    bids: PublicKey;
    asks: PublicKey;
    eventQueue: PublicKey;
    quoteLotSize: BN;
    baseLotSize: BN;
    impactQuantity: BN;
    baseDecimals: number;
    registrationTime: BN;
    name: string;
    maintAssetWeight: I80F48;
    initAssetWeight: I80F48;
    maintLiabWeight: I80F48;
    initLiabWeight: I80F48;
    liquidationFee: I80F48;
    makerFee: I80F48;
    takerFee: I80F48;
    minFunding: I80F48;
    maxFunding: I80F48;
    openInterest: number;
    seqNum: number;
    feesAccrued: I80F48;
    priceLotsToUiConverter: number;
    baseLotsToUiConverter: number;
    quoteLotsToUiConverter: number;
    price: number;
    uiPrice: number;
    static from(publicKey: PublicKey, obj: {
        group: PublicKey;
        quoteTokenIndex: number;
        perpMarketIndex: number;
        name: number[];
        oracle: PublicKey;
        oracleConfig: OracleConfig;
        bids: PublicKey;
        asks: PublicKey;
        eventQueue: PublicKey;
        quoteLotSize: BN;
        baseLotSize: BN;
        maintAssetWeight: I80F48Dto;
        initAssetWeight: I80F48Dto;
        maintLiabWeight: I80F48Dto;
        initLiabWeight: I80F48Dto;
        liquidationFee: I80F48Dto;
        makerFee: I80F48Dto;
        takerFee: I80F48Dto;
        minFunding: I80F48Dto;
        maxFunding: I80F48Dto;
        impactQuantity: BN;
        longFunding: I80F48Dto;
        shortFunding: I80F48Dto;
        fundingLastUpdated: BN;
        openInterest: BN;
        seqNum: any;
        feesAccrued: I80F48Dto;
        bump: number;
        baseDecimals: number;
        registrationTime: BN;
    }): PerpMarket;
    constructor(publicKey: PublicKey, group: PublicKey, quoteTokenIndex: number, perpMarketIndex: number, name: number[], oracle: PublicKey, oracleConfig: OracleConfig, bids: PublicKey, asks: PublicKey, eventQueue: PublicKey, quoteLotSize: BN, baseLotSize: BN, maintAssetWeight: I80F48Dto, initAssetWeight: I80F48Dto, maintLiabWeight: I80F48Dto, initLiabWeight: I80F48Dto, liquidationFee: I80F48Dto, makerFee: I80F48Dto, takerFee: I80F48Dto, minFunding: I80F48Dto, maxFunding: I80F48Dto, impactQuantity: BN, longFunding: I80F48Dto, shortFunding: I80F48Dto, fundingLastUpdated: BN, openInterest: BN, seqNum: BN, feesAccrued: I80F48Dto, bump: number, baseDecimals: number, registrationTime: BN);
    loadAsks(client: MangoClient): Promise<BookSide>;
    loadBids(client: MangoClient): Promise<BookSide>;
    loadEventQueue(client: MangoClient): Promise<PerpEventQueue>;
    loadFills(client: MangoClient, lastSeqNum: BN): Promise<(OutEvent | FillEvent | LiquidateEvent)[]>;
    /**
     *
     * @param bids
     * @param asks
     * @returns returns funding rate per hour
     */
    getCurrentFundingRate(bids: BookSide, asks: BookSide): number;
    uiPriceToLots(price: number): BN;
    uiBaseToLots(quantity: number): BN;
    uiQuoteToLots(uiQuote: number): BN;
    priceLotsToUi(price: BN): number;
    baseLotsToUi(quantity: BN): number;
    quoteLotsToUi(quantity: BN): number;
    toString(): string;
}
export declare class BookSide {
    client: MangoClient;
    perpMarket: PerpMarket;
    type: BookSideType;
    bumpIndex: any;
    freeListLen: any;
    freeListHead: any;
    rootNode: any;
    leafCount: any;
    nodes: any;
    includeExpired: boolean;
    private static INNER_NODE_TAG;
    private static LEAF_NODE_TAG;
    now: BN;
    static from(client: MangoClient, perpMarket: PerpMarket, bookSideType: BookSideType, obj: {
        bumpIndex: number;
        freeListLen: number;
        freeListHead: number;
        rootNode: number;
        leafCount: number;
        nodes: unknown;
    }): BookSide;
    constructor(client: MangoClient, perpMarket: PerpMarket, type: BookSideType, bumpIndex: any, freeListLen: any, freeListHead: any, rootNode: any, leafCount: any, nodes: any, includeExpired?: boolean, maxBookDelay?: number);
    static getPriceFromKey(key: BN): BN;
    items(): Generator<PerpOrder>;
    getImpactPriceUi(baseLots: BN): number | undefined;
    getL2(depth: number): [number, number, BN, BN][];
    getL2Ui(depth: number): [number, number][];
    static toInnerNode(client: MangoClient, data: [number]): InnerNode;
    static toLeafNode(client: MangoClient, data: [number]): LeafNode;
}
export declare class BookSideType {
    static bids: {
        bids: {};
    };
    static asks: {
        asks: {};
    };
}
export declare class LeafNode {
    ownerSlot: number;
    orderType: PerpOrderType;
    timeInForce: number;
    key: BN;
    owner: PublicKey;
    quantity: BN;
    clientOrderId: BN;
    timestamp: BN;
    static from(obj: {
        ownerSlot: number;
        orderType: PerpOrderType;
        timeInForce: number;
        key: BN;
        owner: PublicKey;
        quantity: BN;
        clientOrderId: BN;
        timestamp: BN;
    }): LeafNode;
    constructor(ownerSlot: number, orderType: PerpOrderType, timeInForce: number, key: BN, owner: PublicKey, quantity: BN, clientOrderId: BN, timestamp: BN);
}
export declare class InnerNode {
    children: [number];
    static from(obj: {
        children: [number];
    }): InnerNode;
    constructor(children: [number]);
}
export declare class Side {
    static bid: {
        bid: {};
    };
    static ask: {
        ask: {};
    };
}
export declare class PerpOrderType {
    static limit: {
        limit: {};
    };
    static immediateOrCancel: {
        immediateorcancel: {};
    };
    static postOnly: {
        postonly: {};
    };
    static market: {
        market: {};
    };
    static postOnlySlide: {
        postonlyslide: {};
    };
}
export declare class PerpOrder {
    orderId: BN;
    clientId: BN;
    owner: PublicKey;
    openOrdersSlot: number;
    feeTier: 0;
    price: number;
    priceLots: BN;
    size: number;
    sizeLots: BN;
    side: Side;
    timestamp: BN;
    expiryTimestamp: BN;
    static from(perpMarket: PerpMarket, leafNode: LeafNode, type: BookSideType): PerpOrder;
    constructor(orderId: BN, clientId: BN, owner: PublicKey, openOrdersSlot: number, feeTier: 0, price: number, priceLots: BN, size: number, sizeLots: BN, side: Side, timestamp: BN, expiryTimestamp: BN);
}
export declare class PerpEventQueue {
    static FILL_EVENT_TYPE: number;
    static OUT_EVENT_TYPE: number;
    static LIQUIDATE_EVENT_TYPE: number;
    head: number;
    count: number;
    seqNum: BN;
    rawEvents: (OutEvent | FillEvent | LiquidateEvent)[];
    constructor(client: MangoClient, header: {
        head: number;
        count: number;
        seqNum: BN;
    }, buf: any);
    getUnconsumedEvents(): (OutEvent | FillEvent | LiquidateEvent)[];
    eventsSince(lastSeqNum?: BN): (OutEvent | FillEvent | LiquidateEvent)[];
}
export interface Event {
    eventType: number;
}
export interface OutEvent extends Event {
    side: PerpOrderType;
    ownerSlot: number;
    timestamp: BN;
    seqNum: BN;
    owner: PublicKey;
    quantity: BN;
}
export interface FillEvent extends Event {
    takerSide: PerpOrderType;
    makerOut: boolean;
    makerSlot: number;
    marketFeesApplied: boolean;
    timestamp: BN;
    seqNum: BN;
    maker: PublicKey;
    makerOrderId: BN;
    makerClientOrderId: BN;
    makerFee: I80F48;
    makerTimestamp: BN;
    taker: PublicKey;
    takerOrderId: BN;
    takerClientOrderId: BN;
    takerFee: I80F48;
    price: BN;
    quantity: BN;
}
export interface LiquidateEvent extends Event {
    seqNum: BN;
}
