import { Market, Orderbook } from '@project-serum/serum/lib/market';
import { Cluster, PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import { MangoClient } from '../client';
import { Group } from './group';
export declare class Serum3Market {
    publicKey: PublicKey;
    group: PublicKey;
    baseTokenIndex: number;
    quoteTokenIndex: number;
    serumProgram: PublicKey;
    serumMarketExternal: PublicKey;
    marketIndex: number;
    registrationTime: BN;
    name: string;
    static from(publicKey: PublicKey, obj: {
        group: PublicKey;
        baseTokenIndex: number;
        quoteTokenIndex: number;
        name: number[];
        serumProgram: PublicKey;
        serumMarketExternal: PublicKey;
        marketIndex: number;
        bump: number;
        registrationTime: BN;
    }): Serum3Market;
    constructor(publicKey: PublicKey, group: PublicKey, baseTokenIndex: number, quoteTokenIndex: number, name: number[], serumProgram: PublicKey, serumMarketExternal: PublicKey, marketIndex: number, registrationTime: BN);
    /**
     *
     * @param group
     * @returns maximum leverage one can bid on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxBidLeverage(group: Group): number;
    /**
     *
     * @param group
     * @returns maximum leverage one can ask on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxAskLeverage(group: Group): number;
    getSerum3ExternalMarket(group: Group): Market | undefined;
    loadBids(client: MangoClient, group: Group): Promise<Orderbook>;
    loadAsks(client: MangoClient, group: Group): Promise<Orderbook>;
    logOb(client: MangoClient, group: Group): Promise<string>;
}
export declare class Serum3SelfTradeBehavior {
    static decrementTake: {
        decrementTake: {};
    };
    static cancelProvide: {
        cancelProvide: {};
    };
    static abortTransaction: {
        abortTransaction: {};
    };
}
export declare class Serum3OrderType {
    static limit: {
        limit: {};
    };
    static immediateOrCancel: {
        immediateOrCancel: {};
    };
    static postOnly: {
        postOnly: {};
    };
}
export declare class Serum3Side {
    static bid: {
        bid: {};
    };
    static ask: {
        ask: {};
    };
}
export declare function generateSerum3MarketExternalVaultSignerAddress(cluster: Cluster, serum3Market: Serum3Market, serum3MarketExternal: Market): Promise<PublicKey>;
