import { AccountMeta, PublicKey, TransactionInstruction } from '@solana/web3.js';
import BN from 'bn.js';
import { Group } from './accounts/group';
import { I80F48 } from './accounts/I80F48';
import { MangoAccount } from './accounts/mangoAccount';
export declare const U64_MAX_BN: BN;
export declare const I64_MAX_BN: BN;
export declare function debugAccountMetas(ams: AccountMeta[]): void;
export declare function debugHealthAccounts(group: Group, mangoAccount: MangoAccount, publicKeys: PublicKey[]): void;
export declare function findOrCreate<T>(entityName: string, findMethod: (...x: any) => any, findArgs: any[], createMethod: (...x: any) => any, createArgs: any[]): Promise<T>;
/**
 * Get the address of the associated token account for a given mint and owner
 *
 * @param mint                     Token mint account
 * @param owner                    Owner of the new account
 * @param allowOwnerOffCurve       Allow the owner account to be a PDA (Program Derived Address)
 * @param programId                SPL Token program account
 * @param associatedTokenProgramId SPL Associated Token program account
 *
 * @return Address of the associated token account
 */
export declare function getAssociatedTokenAddress(mint: PublicKey, owner: PublicKey, allowOwnerOffCurve?: boolean, programId?: PublicKey, associatedTokenProgramId?: PublicKey): Promise<PublicKey>;
export declare function createAssociatedTokenAccountIdempotentInstruction(payer: PublicKey, owner: PublicKey, mint: PublicKey): Promise<TransactionInstruction>;
export declare function toNative(uiAmount: number, decimals: number): I80F48;
export declare function toNativeDecimals(amount: number, decimals: number): BN;
export declare function toUiDecimals(amount: I80F48 | number, decimals: number): number;
export declare function toUiDecimalsForQuote(amount: I80F48 | number): number;
export declare function toU64(amount: number, decimals: number): BN;
export declare function nativeI80F48ToUi(amount: I80F48, decimals: number): I80F48;
